log using wmHS.NLSY.79_93X5.log, text replace

* wmHS.NLSY.79_93X5f.input.do
* copied from wmHS.NLSY.79_93X5.input.do

* final version

* record previous location at age 20 (if any)
* include only cross-section observations (X)
* include all home locations found in the data
* delete people who ever enrolled in college
* 
* history from age 20

 pause on
* pause off (default)

set linesize 150

set matsize 90
set more off
drop _all
set memory 190m


pause on

* migration histories 
* earnres8L includes extract through 1994
* earnres9L has an improved age variable (year-birthyr if birthmo <= 6, year-birthyr-1 if birthmo>6)
use \jw\data\earnres9L.dta, clear

* first select people, then select years

			* 1	Cross	Male	White		2236
			* 2	Cross	Male	Wh.	Poor	 203
			* 3	Cross	Male	Black		 346
			* 4	Cross	Male	Hispanic	 218
drop if smp == 3	
drop if smp == 4	

*  5	Cross	Female	White		2279
*  6	Cross	Female	Wh.	Poor	 198
*  7	Cross	Female	Black		 405
*  8	Cross	Female	Hispanic	 226
drop if smp == 5
drop if smp == 6	
drop if smp == 7	
drop if smp == 8	
					
			* 9	Sup	Male	Wh.	Poor	 742
			* 10	Sup	Male	Black		1105
			* 11	Sup	Male	Hispanic	 729
drop if smp ==  9
drop if smp == 10
drop if smp == 11

* 12	Sup	Fem	Wh	Poor	 901
* 13	Sup	Female	Black		1067
* 14	Sup	Female	Hispanic 	 751
drop if smp == 12
drop if smp == 13	
drop if smp == 14

			* 15	Military	Male	White		609
			* 16	Military	Male	Black		162
			* 17	Military	Male	Hispanic 	 53
			* 18	Military	Female	White		342
			* 19	Military	Female	Black		 89
			* 20	Military	Female	Hispanic	 25
drop if smp == 15	
drop if smp == 16	
drop if smp == 17	
drop if smp == 18	
drop if smp == 19	
drop if smp == 20	

sort caseid year
by caseid: gen obsno  = _n

count if obsno == 1
count if year > 78

* redo the highschool variables to include information for 93 and 94 

*R02166.01    [*Created]  ENROLLMENT STATUS AS OF MAY 1 SURVEY YEAR (REVISED)

*    1866       1 NOT ENROLLED, COMPLETED LESS THAN 12TH GRADE
*    5745       2 ENROLLED IN HIGH SCHOOL
*    1681       3 ENROLLED IN COLLEGE
*    3388       4 NOT ENROLLED, HIGH SCHOOL GRADUATE
*   -----
*   12680


* identify people who ever report being on active duty in the military
* (see NLSY guide, http://www.bls.gov/nls/79guide/2001/nls79g4b.pdf, p237)
 gen byte military_ = (cps==-4 | mil ==1)
egen byte military  = sum(military_), by(caseid)

pause

drop if military > 0
	count if obsno==1

drop maxhgc
egen maxhgc = max(hgc), by(caseid)
drop hsgrad
gen hsgrad = (enr==4 & maxhgc == 12)
label var hsgrad "HS grad, enr=4,maxenr==4,maxhgc==12"
tab hgc if hsgrad == 1


drop maxenr
egen maxenr=max(enr), by(caseid)

pause

* drop people who ever report attending college (post hs)
 gen byte cenroll_ = enr==3
egen byte cenroll  = sum(cenroll_), by(caseid)


*keep high school graduates only
keep if maxhgc == 12
	count if obsno==1
drop if cenroll>0
	count if obsno==1

pause

drop if  birthyr == 57
	count if obsno==1

drop if  birthyr == 58 & birthmo < 7
	count if obsno==1


*keep only people who graduate high school by age 20 (and get no more education)  
gen int hgc20_ = hgc<12 & age>=20 & hgc>0
egen hgc20_max = max(hgc20_),by(caseid)

pause

gen int hgc20y = hgc==12 & age <=20 & age > 0
egen hgc20ymax = max(hgc20y),by(caseid)


* drop if afqtp80 is missing 
 drop if afqtp80 < 0 
	count if obsno==1

pause

drop if hgc20ymax == 0
	count if obsno==1
keep if hgc20_max==0
	count if obsno==1

drop if maxenr < 4

pause 

 gen byte inschool20 = esr == 5 & age > 19
egen byte Inschool20 = max(inschool20) , by(caseid)
  drop if Inschool20 ==1

	count if obsno==1


egen int esrmax = max(esr) if age > 19, by(caseid)
 gen byte    lf = esr    < 4
 gen byte   L_F = esrmax < 4
egen int L_F2 = max(L_F) , by(caseid)

egen int fbmax = max(fbus), by(caseid)
 gen byte   FB = fbmax > 0

pause

 gen byte unable20 = esr == 6 & age > 19
egen int  Unable20 = sum(unable20) , by(caseid)
drop if Unable20 > 1

	count if obsno==1


 gen byte house20 = esr == 4 & age > 19
egen int  House20 = sum(house20) , by(caseid)
drop if House20 > 1

	count if obsno==1


 gen byte other20 = esr == 7 & age > 19
egen int  Other20 = sum(other20) , by(caseid)
drop if Other20 > 1

	count if obsno==1


 gen byte nlf20 = esr >3 & age > 19
egen int  Nlf20 = sum(nlf20) , by(caseid)
drop if   Nlf20 > 1

	count if obsno==1


 gen byte nlf0 = esr >3 & age == 20
egen int  Nlf0 = sum(nlf0) , by(caseid)
drop if Nlf0 == 1

	count if obsno==1


 gen byte nostart = drni > 1 & age >19 & age < 22
egen int  Nostart = sum(nostart) , by(caseid)
drop if Nostart > 0

 count if obsno==1

gen gap00 = ( drni > 1 | totws < 0 | fbus==-1 | fbus==-2| fbus==-3 | (totws+fbus==0&(hrs>0|wks>0)) ) & age > 19 
gen good = 1-gap00 if age > 19
egen Good = sum(good), by(caseid)
drop if Good == 0

 count if obsno==1

 gen int dead = drni == 65 & age < 30
egen int Dead = sum(dead), by(caseid)
drop if Dead > 0

 count if obsno==1

* conflict between residence at age 14 and living in the same residence since birth -- probably a coding error
replace resa14=25 if caseid==11750

* missing residence at age 14, but living in the same residence since birth
gen int rstate = state if R0003000 == 1 &  resa14==. & year == 79
egen Rstate = max(rstate), by(caseid)
replace resa14 = Rstate if resa14 == .

tab esrlab L_F2

pause 

*eliminate those without a "home location"
drop if resa14==.
	count if obsno==1

*drop people who are not known to have graduated high school by age 20  

* #526 was in jail in 1993
drop if caseid==526

 count if obsno==1
 count
 count if year > 78

tab year

** finished selecting people: now select years

* recode DC as MD (just one observation)

pause

replace state  = 24         if state  == 11
replace sname = "MARYLAND" if sname == "DC"

pause

*eliminate observations from 1978 (income but no state information)
drop if year == 78

count
dis (93-78)*439

tsset caseid year
format year %6.0g

gen dsta = F.state

* generate continuous history, from age 20 (known origin and destination, in the labor force, valid wage information)
    gen gap0 = (dsta < 0 | drni > 1 | esr > 3 | totws < 0 | fbus==-1 | fbus==-2| fbus==-3 | (totws+fbus==0&(hrs>0|wks>0)) ) & age > 19
replace gap0 = 1 if fbus == -4 & totws==0 & (hrs>0 | wks>0) & age > 19 
egen ltt_ = min(obsno) if gap0==1, by(caseid)
egen ltt  = max(ltt_), by(caseid)

pause
count if obsno >= ltt & year < 94
drop if obsno >= ltt

egen byte firstyear = min(year), by(caseid)
gen  byte obs1=year==firstyear

count if obs1==1
count

drop if age<20
count

pause

egen minage= min(age), by(caseid)
drop if minage > 20

count
pause
drop if year==94

drop if totws < 0

drop if drni > 1

gen byte move = (state != dsta & dsta != .)
gen byte moved = state!=state[_n-1] & caseid==caseid[_n-1]

drop if state == 7 | state == 3 | state == 14 | state == 43 | state == 52

* generate previous state
tsset caseid year
gen byte L1state = L.state      if L.state   != state & L.state > 0
gen byte L2state = L.L1state    if L.L1state != L1state &  L.L1state > 0
gen byte L3state = L.L2state    if L.L2state != L2state &  L.L2state > 0
gen byte L4state = L.L3state    if L.L3state != L3state &  L.L3state > 0
gen byte L5state = L.L4state    if L.L4state != L4state &  L.L4state > 0
gen byte L6state = L.L5state    if L.L5state != L5state &  L.L5state > 0
gen byte L7state = L.L6state    if L.L6state != L6state &  L.L6state > 0
gen byte L8state = L.L7state    if L.L7state != L7state &  L.L7state > 0
gen byte L9state = L.L8state    if L.L8state != L8state &  L.L8state > 0
gen byte L10state = L.L9state   if L.L9state != L9state &  L.L9state > 0
gen byte L11state = L.L10state  if L.L10state != L10state & L.L10state > 0
gen byte L12state = L.L11state  if L.L11state != L11state & L.L11state > 0
gen byte L13state = L.L12state  if L.L12state != L12state & L.L12state > 0
gen byte L14state = L.L13state  if L.L13state != L13state & L.L13state > 0
gen byte L15state = L.L14state  if L.L14state != L14state & L.L14state > 0
gen psta = L1state 

count

sort caseid year

*keep old coding for psta 
replace psta = L2state if psta == .
replace psta = L3state if psta == .
replace psta = L4state if psta == .
replace psta = L5state if psta == .
replace psta = L6state if psta == .
replace psta = L7state if psta == .
replace psta = L8state if psta == .
replace psta = L9state if psta == .
replace psta = L10state if psta == .
replace psta = L11state if psta == .
replace psta = L12state if psta == .
replace psta = L13state if psta == .
replace psta = L14state if psta == .
replace psta = L15state if psta==.
replace psta = 0 if psta == state
replace psta = 0 if psta == .

drop  L?state
drop L??state

pause


egen byte nmove = sum(move), by(caseid)
egen byte year1 = min(year), by(caseid)
gen  byte firstobs=year==year1
egen evermove = max(move), by(caseid)

by caseid: gen byte movenum = sum(move)
replace movenum=0 if move==0

tab firstobs
pause

tab evermove if firstobs==1
egen moves = sum(move), by(caseid)
tab moves if firstobs==1

tab move

*create earnings
*income data from created variables on income previous calendar year
*no previous calendar income available for even number years starting in 1994 (switched to biennial survey)
 
replace rtotws = totws/cpi if year == 93 
replace rtotws = rtotws*(100001/totws) if totws > 100000.
replace rtotws = rtotws*( 75001/totws) if totws >  75000. & year <= 85

* assume farm and business income is zero in the case of a valid skip
replace fbus = 0 if fbus ==-4
* replace rtotws = . if totws+fbus == 0 & (wks > 0 | hrs > 0)
* replace rtotws = . if totws < 0
* this includes refusals and "don't know"

drop if dsta == .
drop if dsta < 0
drop if dsta > 56

drop if psta == .
drop if psta < 0
drop if psta > 56

drop if state < 0
drop if state > 56

drop if esr > 3


tab firstobs
pause

sort state
merge state using \jw\data\ycolaWM.dta
list state if _merge == 2,noobs
drop if _merge == 2
drop _merge

replace staten = "MARYLAND" if staten == "DC"

* define cola adjusted real earnings
gen rinc1 = (rtotws + fbus/cpi)/cola 
replace rinc1 = . if state < 0 | state > 56
count if rinc1 == 0

tab psta
tab year
tab age

sort caseid state
egen cnt = count(year), by(caseid state)
 tab cnt

egen cnt2 = count(year), by(caseid)
 tab cnt2

format %8.0f caseid
format %4.0f resa14 age year cnt
format %3.0f state dsta psta
format %8.0f rinc1

sort caseid year

tab move


outfile caseid resa14 year state  rinc1 psta dsta age afqtp80 afqtp89 using "wmHS.Cola.input.79_93X5f.dat"    ,  replace wide

pause

drop athome
gen byte athome = state==resa14
gen byte athom1 = athome==1 & firstobs==1
egen byte startathome = sum(athom1), by(caseid)
tab startathome
tab firstobs startathome
gen byte movehome = dsta==resa14 & move==1
tab movehome
drop if caseid==.
tab  startathome movehome
tab firstobs startathome
sort caseid year
* list  caseid year age resa14 sname state dsta movehome if move==1 &  startathome==0, noobs
tab movehome if athome==0
tab movehome if athome==0 & psta==resa14
tab movehome if athome==0 & psta!=resa14
tab movehome if athome==0 & psta==0
tab movehome if athome==0 & psta==0 & firstobs==1
tab move if athome==0 & psta==0 & firstobs==1
* list  caseid year age resa14 sname state psta dsta movehome if move==1 &  startathome==0, noobs clean
* list  caseid year age resa14 sname state psta dsta movehome if move==1 &  startathome==0 & movehome==0, noobs clean
tab psta if firstobs==1
tab moves if startathome==1 & firstobs==1
tab moves if startathome==0 & firstobs==1

save "wmHS.Cola.input.79_93X5f.dta", replace

log close
set more on

