SUBROUTINE GIH(qe,Grd,VCM,fval)
!use nrtype; 
use nrutil; use mbr_vars; use mpi
IMPLICIT NONE

Interface
	Function Func(x)
		!use nrtype; 
		use mbr_vars
		IMPLICIT NONE
		Double Precision, intent(in), dimension(:) :: x
		Double Precision :: Func
	end Function Func
end Interface

Interface
	Subroutine ludcmp(a,indx,d)
		!Use nrtype; 
		use nrutil, only : assert_eq,imaxloc,nerror,outerprod,swap
		IMPLICIT NONE
		Double Precision, dimension(:,:), intent(inout) :: a
		Integer, dimension(:), intent(out) :: indx
		Double Precision, intent(out)                   :: d
	end subroutine ludcmp
end interface

Interface
	Subroutine lubksb(a,indx,b)
		!use nrtype; 
		use nrutil, only : assert_eq
		IMPLICIT NONE
		Double Precision, dimension(:,:), intent(in) :: a
		Integer, dimension(:), intent(in) :: indx
		Double Precision, dimension(:), intent(inout)   :: b
	end subroutine lubksb
end interface

	Double Precision, intent(in),  dimension(:)   :: qe !estimated parameters 
	Double Precision, intent(out), dimension(:)   :: Grd  !Gradient
	Double Precision, intent(out), dimension(:,:) :: VCM !Inverse Hessian
	Double Precision, intent(inout)               :: fval

! Calculates numerical approximation of gradient g and outer product
! approximation to estimate Hessian
! Returns Gradient Grd, Inverse Hessian, and function value
! Dennis and Schnabel (1996) recommend setting the stepsize, h, for the
! function evaluation as sqrt(macheps)*typx(x), where typx(x) is the "typical"
! value of the parameter x.  macheps is the smallest number such that 
! 1+macheps > 1. This formula assumes that the function has optimized to 
! precision macheps. Taking the sqrt of macheps gives precision of the left-
! half of the digits.
! The instrinsic function EPSILON() returns macheps, about 1.e-16 and yields
! very small (probably too small) step sizes.

! 7/22/01 include trap for coefficients at zero. Assumes scaling of variables
! so estimated coefficients will be within the unit interval (takes midpt).
!
	Integer		:: i, k, itype, obs_i, ierr
	Double Precision    :: h, tsize
	Double Precision , parameter :: macheps = EPSILON(qe(1)) !est parameters same type
	Double Precision , allocatable, dimension(:,:) :: qdir,  J, HESS
!	Double Precision , dimension(NPARME,NPARME) :: qdir
!	Double Precision , dimension(N_i,0:NPARME)  ::  G
!	Double Precision , dimension(N_i,  NPARME)  ::  J
!	Double Precision , dimension(NPARME,NPARME) ::  HESS
	Double Precision , allocatable, dimension(:)   :: pt, b
	Double Precision , dimension(0:NPARME)         :: y      !function values
	Integer , dimension(:), allocatable :: indx

	if ( (size(qe) .NE. size(Grd)) .OR. (size(qe) .NE. size(VCM,dim=1)) ) then
		print *,"Error:  Matrices in GIH do not conform."
		print *,"   Should equal the value of NPARME, which is ",NPARME
		print *,"   size of qe=",size(qe)
		print *,"   size of Grd=",size(Grd)
		print *,"   size of VDM=",size(VCM,dim=1)
		print *,"Kill job."
		stop
	end if 

	allocate(qdir(NPARME,NPARME))  !needed to build basis for amoeba
!	allocate(G(N_i,0:NPARME)) ! hold function evaluations
		print *,"allocate(J(N_i,1:NPARME))"
	if (.not. allocated(J)) allocate(J(N_i,1:NPARME)) ! grad of ind obs

	if (.not. allocated(f0    )) allocate(f0    (N_i))
	if (.not. allocated(f0save)) allocate(f0save(N_i))

	if (.not. allocated(pt)) allocate(pt(1:NPARME)) 

!	set up direction matrix 
	qdir = 0.0
	forall(i=1:NPARME) qdir(i,i) = 1.0

	y(0)   = fval	!must be loglikelihood (not -loglikelihood value)
	f0save = f0		!available from last call to Func during optimization

	print *," " 
	print *,"GIH:  macheps=",macheps

	do i =1,NPARME - NprobE
		if (abs(qe(i)) < TINY) then !trap coef = 0
			tsize = 0.5 !assumed scaling to within unit interval
		else 
			tsize = abs(qe(i))*max(ffe(i),1.0d0)
		end if
		h      = SQRT(macheps)*tsize  !stepsize
		pt(:)  = qe + qdir(i,:)*h
		y(i)   = -Func( pt(:) )
!		G(:,i) = f0(:) !f0 evaluated in Func
	   	J(:,i) = (f0 - f0save) / h 
		Grd(i) = sum(J(:,i))
		print *,"Parameter            Sum Gradient                Step Size"
		print '(I8,1X,E24.12,1X,E24.12)', i,Grd(i),h
        if (abs(Grd(i)) < TINY2) then
            print *, "zero gradient"
            CALL MPI_ABORT(MPI_COMM_WORLD,-1,ierr)
			stop
		end if
	end do !i

	! compute gradient for mixing probability parameters
	i = NPARME - NprobE
	do itype = NTYPE -(NprobE-1),NTYPE
		i = i + 1
		if (i < 1) exit
		Grd(i) = 0
		do obs_i = 1,N_i
			J(obs_i,i) = (f00(obs_i,itype) - f00(obs_i,1))/L0(obs_i)
		end do ! obs_i
		Grd(i) = sum(J(:,i))
		print *,"Parameter            Sum Gradient                Step Size"
		print '(I8,1X,E24.12,1X,E24.12)', i,Grd(i),h
	end do !itype


	print *," "
!	calculate outer product approximation of Hessian
!	deallocate(G)
	allocate(HESS(NPARME,NPARME))
!	allocate(VCM(NPARME,NPARME))
	if (.not. allocated(indx)) allocate(indx(NPARME))
	if (.not. allocated(b))    allocate(b(NPARME))
	HESS = matmul(transpose(J),J)
	deallocate(J)
	Call ludcmp(HESS,indx,h) !on output HESS contains LU decomposition
	do k=1,NPARME
		b = 0.0
		b(k) = 1.0
		Call lubksb(HESS,indx,b)
		VCM(:,k) = b !build inverse column by column
	end do
	print *," "
!	print *,"Inverse Hessian (Variance-Covariance Matrix): "
!	print '(5(E14.8,1X))',VCM
	forall(k=1:NPARME) b(k) = VCM(k,k)
	print *," "
	print *,"Standard Errors: "
	print '(5(E14.8,1X))',sqrt(b)
end Subroutine GIH 
