 FUNCTION Like_i(obs_i)

! for mbr83W
! calculate the likelihood of an individual history (including wages and migration decisions)

! given: vloc(:) a list of locations visited (maximum is MvLoc, which is set in mbr_vars)
!        position of current and previous locations in this list, in each year (1:13)
!		 wage bin probabilities for each location in the list
!        number of observations for this individual

use mbr_vars
IMPLICIT NONE

Interface
   FUNCTION alnorm( x, upper ) RESULT( fn_val )
     IMPLICIT NONE
     Double Precision, INTENT(IN)	::  x
     LOGICAL,   INTENT(IN)			::  upper
     Double Precision				::  fn_val
   end Function alnorm
end Interface

Integer , intent(in)               :: obs_i

Double Precision, parameter :: phi0 = 0.398942280444 ! 1/sqrt(2*Pi)

Double Precision :: Like_i,  L_i, z,mu_12, w_it, b_it,  y_it, L_w_i_t

Integer :: i_bin,i_bin2,obs_t, cbin, cbin2, pbin, pbin2, fe_bin,i_sig_eps, year0

Integer								:: Nobs_i, Nloc_i

	Integer , allocatable, dimension(:) :: csta_, year_
!	Integer , allocatable, dimension(:) :: loci_index	! index for  current location
!	Double Precision , allocatable, dimension(:) :: wage

	Real , allocatable, dimension(:) ::   age_


	Nobs_i = dat(obs_i)%Nobs_i
	Nloc_i = dat(obs_i)%Nloc_i

!	allocate( csta_( Nobs_i), age_( Nobs_i), loci_index( Nobs_i), wage( Nobs_i), year_( Nobs_i))
	allocate( csta_( Nobs_i), age_( Nobs_i),                                     year_( Nobs_i))

	year0 = 79
	year_ = dat(obs_i)%year(1:Nobs_i) - year0
	csta_ = dat(obs_i)%csta(1:Nobs_i)
	age_  = dat(obs_i)%age (1:Nobs_i)/a0

	Pcb  = 1.0d0/dat(obs_i)%Nbins_i
	Pcb2 = 1.0d0/dat(obs_i)%Nbins2_i

	Like_iw0 = 0.0d0
	Like_i   = 0.0d0

!	if (obs_i == 1) print '(A/)', "dat(obs_i)%id,obs_t, upsilon(cbin) , eta(fe_bin) ,sdat(dat(obs_i)%csta(obs_t))%mu,sig_eps(i_sig_eps), z,L_i,dat(obs_i)%vv(obs_t,cbin,cbin2,pbin,pbin2)"
!	if (obs_i == 2 .AND. plflag == 1) print '(A/)', "obs_t, upsilon(cbin) , eta(fe_bin) ,sdat(dat(obs_i)%csta(obs_t))%mu,sig_eps(i_sig_eps), z,L_i,dat(obs_i)%vv(obs_t,cbin,cbin2,pbin,pbin2)"

	do i_sig_eps = 1, NPT_sig_eps ! transient effect normal mixture distribution
		do fe_bin = 1, NPT_fe		! fixed effect
			do i_bin = 1, dat(obs_i)%Nbins_i
				do i_bin2 = 1, dat(obs_i)%Nbins2_i	

					L_i   = 1.0d0
					L_w_i = 1.0d0
					do obs_t = 1,Nobs_i

						cbin  =  spt( dat(obs_i)%loci_index(obs_t),i_bin)
						cbin2 = spt2( dat(obs_i)%loci_index(obs_t),i_bin2)
						if ( dat(obs_i)%psta(obs_t) > 0 .AND. dat(obs_i)%ploci_index(obs_t) > 0) then
							pbin  =  spt(dat(obs_i)%ploci_index(obs_t),i_bin)
							pbin2 = spt2(dat(obs_i)%ploci_index(obs_t),i_bin2)
						else
							pbin  = 0
							pbin2 = 0
						end if
						L_i = L_i*dat(obs_i)%vv(obs_t,cbin,cbin2,pbin,pbin2)

						w_it = dat(obs_i)%wage(obs_t)
!						if (w_it <= 0) cycle		! ignore missing observations, and treat zero as if the wage is missing
						b_it = Sdat(dat(obs_i)%csta(obs_t))%Ben
						y_it  = max(w_it,b_it)
						mu_12 = mu0   + q(p_mu  )*sdat(dat(obs_i)%csta(obs_t))%mu
						mu_12 = mu_12 + q(p_year)*year_(obs_t) 
						
						mu_12 = mu_12 + q(p_linearwage)*age_(obs_t) + q(p_quadwage)*age_(obs_t)*age_(obs_t)
						mu_12 = mu_12 + (q(p_afqt) + q(p_afqt_age)*age_(obs_t))*dat(obs_i)%afqt/ab0
						z     = (y_it - mu_12 - upsilon(cbin) - eta(fe_bin) )/sig_eps(i_sig_eps)
						if (w_it + TINY2 < b_it) then
							L_w_i_t = alnorm(z, .false.) ! upper = .false. means Phi(z) rather than 1-Phi(z)
						else
							L_w_i_t = phi0*exp(-z*z/2.0d0)/sig_eps(i_sig_eps)
						end if
							L_i   =   L_i*L_w_i_t
							L_w_i = L_w_i*L_w_i_t
!						IF (ISNAN(L_i)) THEN 
!							print '(A/)', "obs_t, upsilon(cbin) , eta(fe_bin) ,sdat(dat(obs_i)%csta(obs_t))%mu,sig_eps(i_sig_eps), z,L_i,dat(obs_i)%vv(obs_t,cbin,cbin2,pbin,pbin2)"
!							print '(I7,10E15.4)', obs_t, upsilon(cbin) , eta(fe_bin) ,sdat(dat(obs_i)%csta(obs_t))%mu,sig_eps(i_sig_eps), z,L_i,dat(obs_i)%vv(obs_t,cbin,cbin2,pbin,pbin2)
!							stop
!						ENDIF ! (ISNAN(F))

						if (L_i < TINY2) then
!							print '(A/)', "dat(obs_i)%id,obs_t, upsilon(cbin) , eta(fe_bin) ,sdat(dat(obs_i)%csta(obs_t))%mu,sig_eps(i_sig_eps), z,L_i,dat(obs_i)%vv(obs_t,cbin,cbin2,pbin,pbin2)"
!							print '(2I7,10E15.4)', dat(obs_i)%id,obs_t, upsilon(cbin) , eta(fe_bin) ,sdat(dat(obs_i)%csta(obs_t))%mu,sig_eps(i_sig_eps), z,L_i,dat(obs_i)%vv(obs_t,cbin,cbin2,pbin,pbin2)
!							print '(A/)',         " obs_t, upsilon(cbin) , eta(fe_bin) ,sdat(dat(obs_i)%csta(obs_t))%mu,sig_eps(i_sig_eps), z,L_i,dat(obs_i)%vv(obs_t,cbin,cbin2,pbin,pbin2)"
!							print '( I7,10E15.4)',  obs_t, upsilon(cbin) , eta(fe_bin) ,sdat(dat(obs_i)%csta(obs_t))%mu,sig_eps(i_sig_eps), z,L_i,dat(obs_i)%vv(obs_t,cbin,cbin2,pbin,pbin2)
!							stop
						end if
						
						if (obs_i == 2 .AND. plflag == 1) then
!							print '(A/)', "dat(obs_i)%id,obs_t, upsilon(cbin) , eta(fe_bin) ,sdat(dat(obs_i)%csta(obs_t))%mu,sig_eps(i_sig_eps), z,L_i,dat(obs_i)%vv(obs_t,cbin,cbin2,pbin,pbin2)"
!							print '(2I7,10E15.4)', dat(obs_i)%id,obs_t, upsilon(cbin) , eta(fe_bin) ,sdat(dat(obs_i)%csta(obs_t))%mu,sig_eps(i_sig_eps), z,L_i,dat(obs_i)%vv(obs_t,cbin,cbin2,pbin,pbin2)
!							print '(A/)',         " obs_t, upsilon(cbin) , eta(fe_bin) ,sdat(dat(obs_i)%csta(obs_t))%mu,sig_eps(i_sig_eps), z,L_i,dat(obs_i)%vv(obs_t,cbin,cbin2,pbin,pbin2)"
							print '( I7,10E15.4)',  obs_t, upsilon(cbin) , eta(fe_bin) ,sdat(dat(obs_i)%csta(obs_t))%mu,sig_eps(i_sig_eps), z,L_i,dat(obs_i)%vv(obs_t,cbin,cbin2,pbin,pbin2)
!							stop
						end if

					end do !obs_t		

					Like_i   = Like_i  + PP_sig_eps(i_sig_eps)*PP1(fe_bin)*Pcb(i_bin)*Pcb2(i_bin2)*L_i
!temporary
!					Like_i   = Like_i  + PP_sig_eps(i_sig_eps)*PP1(fe_bin)*Pcb(i_bin)*Pcb2(i_bin2)*L_w_i

					Like_iw0 = Like_iw0 + PP_sig_eps(i_sig_eps)*PP1(fe_bin)*Pcb(i_bin)*Pcb2(i_bin2)*L_w_i
!                	if (obs_i == 1) print '(2I7,10E15.4)', dat(obs_i)%id,obs_t, upsilon(cbin) , eta(fe_bin) ,sdat(dat(obs_i)%csta(obs_t))%mu,sig_eps(i_sig_eps), z,L_i,dat(obs_i)%vv(obs_t,cbin,cbin2,pbin,pbin2)
!                	if (obs_i == 1) print '( I7,10E15.4)',                      upsilon(cbin) , eta(fe_bin) ,sdat(dat(obs_i)%csta(obs_t))%mu,sig_eps(i_sig_eps), z,L_i,dat(obs_i)%vv(obs_t,cbin,cbin2,pbin,pbin2)
				end do ! i_bin2
			end do ! i_bin
		end do ! fe_bin
	end do ! i_sig_eps 

!
    if ((plflag == 1 .and. obs_i < 51) .OR. Like_i < TINY2) then
		print '(2(A,I7),A,E15.8,A,E15.8)', "obs_i =", obs_i, "Nobs_i =", Nobs_i, " Like_i =", Like_i, " Like_iw0 =", Like_iw0
    end if ! (L_i < TINY2) then

if (obs_i == N_i)  plflag = 0
!
!	deallocate( csta_, age_, loci_index, wage, year_)
	deallocate( csta_, age_,                   year_)

end FUNCTION Like_i
