SUBROUTINE Newton(qe,fval)

! This version uses a simple line search instead of taking a full Newton step

! use nrtype; 
use mbr_vars
IMPLICIT NONE

Interface
	Subroutine GIH(qe,Grd,VCM,fval0)
	!use nrtype; 
	use mbr_vars
	IMPLICIT NONE
		Double Precision , intent(in)  , dimension(:) :: qe    !estimated parameters
		Double Precision , intent(out) , dimension(:) :: Grd   !Gradient NPARME x 1
		Double Precision , intent(out) , dimension(:,:) :: VCM !var-cov matrix 
		Double Precision , intent(inout)                 :: fval0 !initial loglikelihood
	end SUBROUTINE GIH
end interface

Interface
	Function Func(qe)
	!use nrtype; 
	use mbr_vars
	IMPLICIT NONE
		Double Precision , intent(in), dimension(:) :: qe
		Double Precision :: Func
	end FUNCTION Func
end Interface


	Double Precision , intent(inout) , dimension(:)  :: qe			!estimated parameters
	Double Precision , intent(inout)                 :: fval		!new function value
	Double Precision                                 :: oldfval		!old function value
	Double Precision                                 :: fvalhalf	!halfstep function value

	Double Precision ::  xhat,  gamma0, fvalhat

	Double Precision :: q0, dq0, S1, S2

	Integer :: i,j,k, n_M ,obs_i
	Double Precision , dimension(:)  , allocatable :: dqe, qe1, oldqe, qehat, ser, Grd, halfstep
	Double Precision , dimension(:,:), allocatable :: VCM 

	if (.not. allocated(dqe))      allocate(     dqe(Size(qe)))
	if (.not. allocated(qe1))      allocate(     qe1(Size(qe)))
	if (.not. allocated(halfstep)) allocate(halfstep(Size(qe)))
	if (.not. allocated(oldqe))    allocate(   oldqe(Size(qe)))
	if (.not. allocated(qehat))    allocate(   qehat(Size(qe)))
	if (.not. allocated(ser))      allocate(     ser(Size(qe)))
	if (.not. allocated(Grd))      allocate(     Grd(Size(qe)))
	if (.not. allocated(VCM))      allocate(     VCM(Size(qe),Size(qe)))

	Grd(:)   = 0.0
	VCM(:,:) = 0.0

	print *," "
!	print *,"Calculate gradient, inverse Hessian (VCM) and next value of loglike."
	Call GIH(qe,Grd,VCM,fval)  !calculate the Gradient and Inverse Hessian
	print *," "
	print *," "
!	print *,"Parm       Estimate              SE            Grad            Step       t"
	print *,"Parm       Estimate              SE            Grad   Next Estimate       t"
	print *,"---------------------------------------------------------------------------"
	dqe(:)  = -matmul(VCM,Grd)
	forall(k=1:NPARME) ser(k) = sqrt(VCM(k,k))

	do k=1,NPARME
	! prevent big changes in parameter values
		if (abs(qe(k))>TINY .AND. abs(dqe(k)/qe(k)) > MaxStep) then
			dqe  = MaxStep*dqe/abs(dqe(k)/qe(k))
		end if 
		if (abs(qe(k)) <= TINY .AND. abs(dqe(k)) > MaxStep) then
			dqe  = MaxStep*dqe/abs(dqe(k))
		end if 
	end do  ! k
	do k=1,NPARME
		print '(I4,4(1X,E15.8), f8.2)',k,qe(k),ser(k),Grd(k),qe(k)-dqe(k),dqe(k)/ser(k)
	end do  ! k
	print *,"---------------------------------------------------------------------------"
	print *, "maximum step ", maxval(abs(dqe))

	! Newton step for the type probability (two types)
	q0  = 1-qe(NPARME)
	n_M = 0
	S1  = 0
	S2  = 0
	do obs_i = 1,N_i
		if ( dat(obs_i)%Moved .EQV. .FALSE. ) then
			S1 = S1 + q0 * (1 - L0(obs_i)) / (1 - q0 + q0 * L0(obs_i))	! never moved
			S2 = S2 + (1 - L0(obs_i)) / (1 - q0 + q0 * L0(obs_i))**2
		else
			n_M = n_M + 1
		end if
	end do ! obs_i
	dq0 = (n_M-S1)/S2
	
	if (NprobE .EQ. 1) then
		print '(A,4(1X,E15.8))',"type probability, step using numerical gradient, exact step ", q0,dqe(NPARME), dq0
	end if

    	oldqe		= qe
	    halfstep	= oldqe - .5*dqe 
	    qe			= qe - dqe

!	    fval should be bigger than oldfval (both are usually negative numbers)
	    oldfval		= fval
	    fval		= -Func( qe )		!new function value
	    f0save		= f0				! save the gradient
	    fvalhalf	= -Func( halfstep ) !another new function value

!       either the full step or the half step should give an improvement
!       otherwise cut the step in half and try again	    
    do while  (fvalhalf < 1.0*oldfval .AND. fval < 1.0*oldfval )
	    dqe = .5*dqe
      	qe	= halfstep
	    fval = fvalhalf
	    halfstep	= oldqe - .5*dqe 
	    qe			= qe - dqe
	    fvalhalf	= -Func( halfstep ) !another new function value
    end do

	gamma0  = (fvalhalf - oldfval)/(fval - oldfval) ! see quadstep.mw
	if (abs(gamma0) > MaxStep*20.0) gamma0 = 20.0*gamma0/abs(gamma0)	! avoid huge steps
	xhat    = 0.5 + 0.25/(2*gamma0-1)
	qehat    = oldqe - xhat*dqe
	fvalhat	= -Func( qehat )		! quadratic approximation

	print *, " "

	print '(A)',               "loglikelihood values:"
	print '(A)',               "--------------------"
	print '(A,f20.10)',        "previous value        ", oldfval
	print '(A,f20.10)',        "full Newton           ",    fval
	print '(A,f20.10)',        "half Newton           ",fvalhalf 
	print '(A,f20.10,A,f10.5)',"quadratic line search ",fvalhat, " stepsize: ", xhat
	print *, " "


	if (fvalhat > 1.0*oldfval .AND. fvalhat > 1.0*fval ) then 
		qe	= qehat
		fval = fvalhat
        if (xhat < 0.8 ) then
            MaxStep = 0.8d0*Maxstep
            print *, "Reducing Maxstep to ", Maxstep
        end if ! stepsize
        if (xhat > 1.25 ) then
            MaxStep = 1.25d0*Maxstep
            print *, "Increasing Maxstep to ", Maxstep
        end if ! stepsize
	elseif (fval > 1.0*oldfval  ) then 
		f0   = f0save		! go back to the gradient for the full step
	else
	qe = halfstep + dqe/4.0
		do k = 3,25
			print *," "
			print *,"updated parameters: "
			print '(5E18.10,/,5E18.10,/,5E18.10)',qe
			fval		= -Func( qe )		!new function value
			if (fval > 1.0*oldfval ) then
				exit ! do loop
			else
				qe = qe + (0.5**k)*dqe
				print *, " "
				print *, "Full Newton step reduces the likelihood: cutting the step size ", k
				print *,"Function change: from ", oldfval, " to ",   fval
				if (k == 20) then
					stop
				end if
			end if
		end do ! k
	end if

	j=1
	do i=1,NPARMS
		if (ss(i) == 1) then
			q(i) = qe(j)
			std(i) = ser(j)
			if (j==NPARME) exit  !skip out of loop
			j = j+1
		end if
	end do


	print *," "
	print *,"Function evaluation (at next step): ",fval
	print *," "

end SUBROUTINE Newton
