!-- saved from http://users.bigpond.net.au/amiller/apstat/as66.f90 -->
!  Algorithm AS66 Applied Statistics (1973) vol.22, no.3

!  Evaluates the tail area of the standardised normal curve
!  from x to infinity if upper is .true. or
!  from minus infinity to x if upper is .false.

! ELF90-compatible version by Alan Miller
! Latest revision - 29 November 2001

FUNCTION alnorm( x, upper ) RESULT( fn_val )
   IMPLICIT NONE
   Double Precision, INTENT(IN)   ::  x
   LOGICAL,   INTENT(IN)		  ::  upper
   Double Precision               ::  fn_val

   !  Local variables
   Double Precision, PARAMETER  ::  zero=0.0, one=1.0, half=0.5, con=1.28
   Double Precision             ::  z, y
   LOGICAL						::  up

   !  Machine dependent constants
   Double Precision, PARAMETER  ::  ltone = 7.0, utzero = 18.66
   Double Precision, PARAMETER  ::  p = 0.398942280444, q = 0.39990348504,   &
                            r = 0.398942280385, a1 = 5.75885480458,  &
                            a2 =  2.62433121679, a3 = 5.92885724438,  &
                            b1 = -29.8213557807, b2 = 48.6959930692, &
                            c1 = -3.8052E-8, c2 = 3.98064794E-4,     &
                            c3 = -0.151679116635, c4 = 4.8385912808, &
                            c5 =  0.742380924027, c6 = 3.99019417011, &
                            d1 =  1.00000615302, d2 = 1.98615381364,  &
                            d3 =  5.29330324926, d4 = -15.1508972451, &
                            d5 =  30.789933034

   up = upper
   z = x
   IF( z < zero ) THEN
      up = .NOT. up
      z = -z
   END IF
   IF( z <= ltone  .OR.  (up  .AND.  z <= utzero) ) THEN
      y = half*z*z
      IF( z > con ) THEN
         fn_val = r*EXP( -y )/(z+c1+d1/(z+c2+d2/(z+c3+d3/(z+c4+d4/(z+c5+d5/(z+c6))))))
      ELSE
         fn_val = half - z*(p-q*y/(y+a1+b1/(y+a2+b2/(y+a3))))
      END IF
   ELSE
      fn_val = zero
   END IF

   IF( .NOT. up ) fn_val = one - fn_val
   RETURN
END FUNCTION alnorm
