FUNCTION Func(qe)
use mpi
use mbr_vars
IMPLICIT NONE

Interface
   Function Like_iw(obs_i)
     use mbr_vars
     IMPLICIT NONE
     Integer , intent(in) :: obs_i
     Double Precision :: Like_iw
   End Function Like_iw
end Interface

   Character(Len=8)			:: date
   Character(Len=10)		:: time
   Character(Len=5)			:: zone
   Integer , dimension(8)	:: tvim

   Double Precision  , intent(in), dimension(:) :: qe
   Double Precision  :: Func

   Integer :: i,j,obs_i, itype
   Integer :: tiloc
   Double Precision  :: ll,ct1,ct2,zero, ct1a, LL_ip
   
   Double Precision  , allocatable, dimension(:):: tL0

	Integer :: status(MPI_STATUS_SIZE)
	Integer :: sender,sent,recvd,ierr,  i_fe ,i_parm
        
!  update q parameter vector for estimated elements

	j=1
	do i=1,NPARMS
		if (ss(i) == 1) then
			q(i) = qe(j)
			if (j==NPARME) exit  !skip out of loop
			j = j+1
		end if
	end do

	if (.not. allocated(f00)  )  allocate(f00(N_i,NTYPE))
	if (.not. allocated(f0)   )  allocate(f0(N_i))
	if (.not. allocated(L0)   )  allocate(L0(N_i))
	if (.not. allocated(tL0)  )  allocate( tL0(N_i))

   f0   = 0.0d0  !individual observation ll function value
   L0   = 0.0d0
   f00  = 0.0d0
   ll   = 0.0d0
   zero = 0.0d0


   if (abs(q(p_pop_pref)) .GT. TINY) then
      print *, "Error: this program doesn't allow interaction between preference shocks and population size"
      stop
   end if

!  three types
   ptype(2) = q(p_probtype2)
   ptype(3) = q(p_probtype3)
	if (ptype(2) + ptype(3)> 1.0) then
		ptype(2) = q(p_probtype2)/(q(p_probtype2)+q(p_probtype3)) - TINY/2.0
		ptype(3) = q(p_probtype3)/(q(p_probtype2)+q(p_probtype3)) - TINY/2.0
		ptype(1) = TINY
	else
		ptype(1) = 1 - ptype(2)- ptype(3)
	end if
   if ( (ptype(2) < 0.0) .OR. (ptype(2) > 1.0) .OR. (ptype(3) < 0.0) .OR. (ptype(3) > 1.0) .OR. (ptype(1) < 0.0) ) then
      func = 1.0d0/TINY
      print *, "Invalid mixture probabilities, types 1,2,3:", (ptype(i), i=1,3)
      return
   end if

   tiloc = NILOC
   if (NoHome .EQ. 1) tiloc=1

   sent  = 0
   recvd = 0


! Syntax
! call MPI_SEND( buffer, count, type, dest, tag, comm,         ierr)
! call MPI_RECV( buffer, count, type, src , tag, comm, status, ierr)


	do j=1,min(numprocs-1,TILOC)
		CALL MPI_SEND(    0,     1,MPI_INTEGER         ,j,0,MPI_COMM_WORLD,ierr)
		CALL MPI_SEND(    q,NPARMS,MPI_DOUBLE_PRECISION,j,0,MPI_COMM_WORLD,ierr)
		CALL MPI_SEND(ptype, NTYPE,MPI_DOUBLE_PRECISION,j,0,MPI_COMM_WORLD,ierr)
		sent = sent+1
		CALL MPI_SEND(sent,1,MPI_INTEGER,j,0,MPI_COMM_WORLD,ierr)
	end do

	CALL DATE_AND_TIME(date,time,zone,tvim)
	ct1  = real(3600*tvim(5) + 60*tvim(6) + tvim(7) + tvim(8)/1000)
	ct1a = ct1

	do while (recvd < TILOC) 

		CALL MPI_RECV(tL0,N_i,MPI_DOUBLE_PRECISION, MPI_ANY_SOURCE, MPI_ANY_TAG, MPI_COMM_WORLD,status,ierr) 
		itype  = status(MPI_TAG)
		sender = status(MPI_SOURCE) 
		if (itype > 0) then
			f00(:,itype) = f00(:,itype) + tL0
		else
			L0 = L0 + tL0 !tL0 = 0 for obs not from hloc
			recvd = recvd+1
			if (sent < TILOC) then 
				sent  = sent+1
				CALL MPI_SEND(sent,1,MPI_INTEGER,sender,0,MPI_COMM_WORLD,ierr)
			end if ! sent < TILOC
		end if ! (itype > 0)
  
	end do !while recvd .lt. TILOC

!	compute likelihood contribution for stayer type (likelihood of observed wages for those who never moved)

	if (NPT_lm > 3 ) then
		print *, "this program assumes the location match wage distribution has less than 3 support points"
		stop 
	end if
	upsilon(1) = -abs(q(p_sigups))
	do i = 2, NPT_lm
		upsilon(i) = upsilon(1) *(NPT_lm - 1 - i)
	end do ! i

	mu0        = q(p_meanwage)
	sig_eps(1) = abs(q(p_sigeps1))
	sig_eps(2) = abs(q(p_sigeps2))
	sig_eps(3) = abs(q(p_sigeps3))
	sig_eps(4) = abs(q(p_sigeps4))

	i_fe   = 1
	i_parm = p_eta1
	do
		eta(i_fe)   = -abs(q(i_parm ))
		eta(i_fe+1) = 0 
		eta(NPT_fe+1-i_fe) = -eta(i_fe)
		if (2*i_fe+1==NPT_fe) exit
		i_fe   = i_fe + 1
		i_parm = i_parm + 1
	end do

	f00(:,Ntype) = 0.0
!	ll_w = 0.0
	do obs_i=1,N_i
!		ll_w = ll_w + log(LL_ip)
		if ( dat(obs_i)%Moved .eqv. .FALSE.) then
			LL_ip  = Like_iw(obs_i)
			f00(obs_i,Ntype) = LL_ip
			L0(obs_i)        = L0(obs_i) + ptype(Ntype)*LL_ip
		end if ! ( dat(obs_i)%Moved .eq. 0)
	end do !obs_i

!	print '(A)', "    id#      L0(obs_i)      (f00(obs_i,itype),itype=1,Ntype)"
	do obs_i=1,N_i
!		print '(I7,4E15.5)', dat(obs_i)%id, L0(obs_i), (f00(obs_i,itype),itype=1,Ntype)
		if (L0(obs_i) > TINY2) then
			f0(obs_i) = log(L0(obs_i)) !needed in Newton
		else
			print '(A,E15.5,A,F8.1)',  " has likelihood ", L0(obs_i), "; loglikelihood for this person set to", lli00
!			stop
			f0(obs_i) = - 100.0
		end if
	end do !obs_i

	ll   = sum(f0)
 
   CALL DATE_AND_TIME(date,time,zone,tvim)
   ct2 = real(3600*tvim(5) + 60*tvim(6) + tvim(7) + tvim(8)/1000)
   print *,"   (func)  function value is (-ll): ",-ll
   print *,"  Wall clock                   ",ct2-ct1," seconds."
	
   func = -ll    !returns -loglike  to min{-loglike}

    plflag = 0 

end FUNCTION func
