!Program: mbr93a.f90
!Date: January 12, 2009

!Source: mbr93

! mbr93a
! 50 locations (as in mbr87)

! mbr93
! full set of regional dummies
! reduced set of locations

! mbr87b
! 10 regional dummies

! mbr87
! trend in wages

! mbr85
! DC treated as if it is a part of MD

! mbr84
! extended mixture of transient components (4 variance support points)

! mbr83Wa
! include effect of AFQT on wages

! mbr81W
! wage components in levels (not logs)

! mbr81
! age effect on wages estimated jointly with migration process

! mbr80
! location match components in preferences

! mbr71
! 3-point fixed effect distribution

! mbr71
! joint likelihood of wages and migration decisions

! mbr70 
! individual wage effect is a state variable

! mbr61.f90
!  income is unaffected by marriage

! mbr60.f90
! parameter 3 = number of kids (<=3) on moving costs
! returns to welfare eligible population - single women with children
! uses pieces of welmig program (Fall 2005) to calculate utility, EVF
! data structure from 2001 (mbr24 series).
!
!-----------------------------------------------------------------------------
! mbr55.f90
! joint distribution of location match components

! mbr51.f90
! reduced cost of moving to adjacent States

! mbr47.f90
! corrects bug in mbr48 (flow utility from climate variables still not handled properly in util_age)

! mbr47.f90
! corrects bug in mbr40-mbr45c (flow utility from climate variables not handled properly in util_age)

! mbr45b.f90
! make sure that the mixture probabilites are between 0 and 1

! mbr45a.f90
! one type never moves

! mbr45.f90
! analytic derivatives with respect to type probabilities

! mbr44.f90
! loop over types, then over individuals, to save space

! mbr43.f90
! three unobserved types

! mbr41.f90
! a previous location with no wage information is treated as a random draw (like a new location)

! mbr40.f90
! Unobserved heterogeneity: unit of observation is an individual history

! mbr_32.f90
! Outer loop on home location
! inner loop on age

! mbr31.f90
! Program assigns probabilistic bins for wage draws.  Stata do file calculates
! bin probabilities. 

! mbr30.f90
!  Program includes age as an implicit state.  Control deck changed. Current
!  version working with mbr26.f90 (age homogeneous) control deck. 

! mbr26.f90
! Program extends points of support from 3 to NPT, control deck 
! simplified by eliminated rho (as rho=0 is required in the EVF 
! routine), and COLA parameter dropped (as cost of living adjustment 
! is done in the data prep [do file] step generating the *.input.dat 
! file.  Changes made in reading the control deck, and the EVF routine.

! mbr21.f90 
! Program automatically adjusts calculation of EVF when the home 
! premium coefficient is zero.  Changed stepsize to admit possible 
! zero coefficients in GIH subroutine.
!------------------------------------------------------------------------------ 
program mbr93a
use mpi 
use nrutil, ONLY : nerror 
use nr
use mbr_vars
IMPLICIT NONE
!
!
interface
   Subroutine fill_miles
     use mpi; use mbr_vars
     IMPLICIT NONE
   end subroutine fill_miles
end interface

Interface
   Function Func(x)
     use mpi; use mbr_vars
     IMPLICIT NONE
     Double Precision, intent(in), dimension(:) :: x
     Double Precision :: Func
   end Function Func
end Interface

Interface
   SUBROUTINE Newton(qe,fval)
     use mpi; use mbr_vars
     IMPLICIT NONE
     Double Precision, intent(inout), dimension(:) :: qe
     Double Precision, intent(inout)               :: fval !loglikelihood value
   end SUBROUTINE Newton
end Interface

Interface 
   SUBROUTINE read_data
     use mpi; use mbr_vars
     IMPLICIT NONE
   end SUBROUTINE read_data
end Interface

Interface
   SUBROUTINE read_climate()
     use mpi; use mbr_vars
     IMPLICIT NONE
   end SUBROUTINE read_climate
end interface

Interface
   SUBROUTINE read_adj  !reads matrix of adjacent states
     use mpi 
     use mbr_vars
     IMPLICIT NONE
   end SUBROUTINE read_adj
end Interface

Interface 
   SUBROUTINE start_values
     use mbr_vars
     IMPLICIT NONE
   end SUBROUTINE start_values
end Interface

Interface 
   SUBROUTINE par_values
     use mbr_vars
     IMPLICIT NONE
   end SUBROUTINE par_values
end Interface

Interface 
   Subroutine Slave_work 
     use mpi
     use mbr_vars
     IMPLICIT NONE
   end Subroutine Slave_work
end Interface

!-----------------------------------------------------------------
Integer :: i, j, iters,  flag, age, i_bin,k,i_loc, statenum
Integer :: tloc,tage,tsl,ios,ierr
Double Precision :: tpop,rben, cwhat2

Integer , dimension(:), allocatable :: loc
Integer , dimension(NLOC) :: sloc!default=0 means incl State (1 means exclude)
!   Character(len=10), dimension(NLOC) :: sname
Character(len=10)  :: tsname
Character(len=80) :: title

Double Precision :: fval00, fval0, fval1, fval
! Double Precision, dimension(NPARMS) :: ff   !scale factors to build basis for amoeba
Double Precision, dimension(:),   allocatable :: qe			!estimated parameter vector
Double Precision, dimension(:,:), allocatable :: qdir !direction vector
Double Precision, dimension(:),   allocatable   :: y
Double Precision, dimension(:,:), allocatable :: pt

!------------------------------------------------------------------

!------------------------------------------------------------------

CALL MPI_INIT(ierr)
CALL MPI_COMM_RANK(MPI_COMM_WORLD,myid,ierr)
CALL MPI_COMM_SIZE(MPI_COMM_WORLD,numprocs,ierr)

if (numprocs .eq. 1) then
   print *,"Error: Program requires multiple processors."
   print *,"Run hosts program and include number of processors on command line."
   CALL MPI_ABORT(MPI_COMM_WORLD,-1,ierr)
   stop
end if

if (myid .eq. master) print *,"Master"


!read control deck and set up q (global) vector
!because of MPI inability to read name of standard in
!control deck hard-coded with file name = control_deck.dat
!this subroutine is executed by all processors

!	read control deck, 
open(unit=17,file=cdeck,STATUS='OLD',ACTION='READ',IOSTAT=ios)
if (ios < 0 .and. myid .eq. master) then
   print *,"ERROR: problem opening ", cdeck
   CALL MPI_ABORT(MPI_COMM_WORLD,-1,ierr)
end if

if (ios /= 0) then
   if (myid .eq. master) then
      print *,"Error in opening ",cdeck
		print *, "IOSTAT = ", ios
      CALL MPI_ABORT(MPI_COMM_WORLD,-1,ierr)
   end if
end if

spt = 1
do i_bin = 2,Nbins
   do k = i_bin,NBINS
      spt(:,k) = spt(:,i_bin-1)
   end do ! k
   do i_loc = 1,Mvloc
      spt(i_loc,i_bin) = spt(i_loc,i_bin-1) + 1
      if (spt(i_loc,i_bin) .LE. NPT_lm) exit  
      spt(i_loc,i_bin) = 1
   end do ! i_loc
end do ! i_bin

spt2 = 1
do i_bin = 2,Nbins2
   do k = i_bin,NBINS2
      spt2(:,k) = spt2(:,i_bin-1)
   end do ! k
   do i_loc = 1,Mvloc
      spt2(i_loc,i_bin) = spt2(i_loc,i_bin-1) + 1
      if (spt2(i_loc,i_bin) .LE. NPT_lm2) exit  
      spt2(i_loc,i_bin) = 1
   end do ! i_loc
end do ! i_bin

!	read control deck, 
	
allocate(loc(NLOC))

!	NPARME = # of parameters to estimate
!	IFAC = scaling parameter 
!	FEV is the Maximum number of Newton Steps to be performed
!	if FEV == 0, the program calculates up to 100 Newton Steps 

FEV = 0   !set default as calculate standard errors and two Newton steps

read(unit=17,FMT='(A)') title
read(unit=17,FMT='(A)') wage_fname  ! expected income data
read(unit=17,FMT='(A)') in_cdname   !mean wage data
read(unit=17,FMT='(A)') in_fname    !name of input dataset, global variable
read(unit=17,FMT='(A)') adj_fn      !full path name of adjacent.dat
read(unit=17,FMT='(A)') amenity_fn  !full path name of amenity.dat
read(unit=17,FMT='(A)') latlong_fn  !full path name of latlong.dat
read(unit=17,FMT=*) !skip 
read(unit=17,FMT=*) !skip over variable names 

read(unit=17,FMT=*) NPARME,IFAC,FEV,BETA, CAP_AGE
do k=1,NPARMS
    read(unit=17,FMT=*,IOSTAT=ios) i, q(i), ss(i), ff(i)  !start value, estimation, scale factor
end do !i 
read(unit=17,FMT=*) !skip 
read(unit=17,FMT=*) !skip 
do
   read(unit=17,FMT=*,IOSTAT=ios) statenum, rben
   if (ios < 0) exit
   Sdat(statenum)%Ben      = rben/IFAC
end do

open(unit=17,file=wage_fname,STATUS='OLD',ACTION='READ',IOSTAT=ios)
if (ios /= 0) then
   print *,"Error:: Problem opening ", wage_fname
   stop
end if
i = 0
maxage = 0 
do
   read(unit=17,FMT=*,IOSTAT=ios) tloc,tsl, tage, tpop,tsname 
   if (ios < 0) exit
   if (tage == FIRST_AGE) then
      i=i+1
   end if
   loc(i) = tloc
   sloc(i) = tsl
   Sdat(i)%pop  = tpop
   Sdat(i)%name = tsname
   maxage = max(tage,maxage)
end do

!	read mean log wages, by state and age
open(unit=3,FILE=in_cdname,STATUS="OLD",IOSTAT=ios)
if (ios /= 0) then
   print *,"data file containing age- and state-specific wages is missing."
   stop
end if

!keep only state mean as of age 20
do
   read(unit=3,FMT=*,IOSTAT=ios) statenum, age,  cwhat2
   if (ios < 0) exit
   Sdat(statenum)%mu  = cwhat2/IFAC
end do

! South
!	Sdat( 1)%region = 1	! Alabama
!	Sdat( 4)%region = 1	! Arkansas
!	Sdat(18)%region = 1	! Lousiana
!	Sdat(24)%region = 1	! Mississippi

!	Sdat( 5)%region = 1	! California

! Northwest
!	Sdat(37)%region = 1	! Oregon
!	Sdat(47)%region = 1	! Washington

! Upper Midwest 
!	Sdat(22)%region = 1	! Michigan
!	Sdat(23)%region = 1	! Minnesota
!	Sdat(34)%region = 1	! North Dakota
!	Sdat(41)%region = 1	! South Dakota
!	Sdat(49)%region = 1	! Wisconsin

! Midwest 
!	Sdat(13)%region = 1	! Illinois
!	Sdat(14)%region = 1	! Indiana
!	Sdat(15)%region = 1	! Iowa
!	Sdat(16)%region = 1	! Kansas
!	Sdat(25)%region = 1	! Missouri
!	Sdat(27)%region = 1	! Nebraska
!	Sdat(35)%region = 1	! Ohio

! East
!	Sdat( 7)%region = 1	! Connecticut
!	Sdat( 8)%region = 1	! Delaware
!	Sdat(19)%region = 1	! Maine
!	Sdat(20)%region = 1	! Maryland
!	Sdat(21)%region = 1	! Massachusetts
!	Sdat(29)%region = 1	! New Hampshire
!	Sdat(30)%region = 1	! New Jersey
!	Sdat(32)%region = 1	! New York
!	Sdat(38)%region = 1	! Pennsylvania
!	Sdat(39)%region = 1	! Rhode Island
!	Sdat(45)%region = 1	! Vermont

! Southeast
!	Sdat( 9)%region = 1	! Florida
!	Sdat(10)%region = 1	! Georgia
!	Sdat(33)%region = 1	! North Carolina
!	Sdat(40)%region = 1	! South Carolina
!	Sdat(46)%region = 1	! Virginia

! Appalachia
!	Sdat(17)%region = 1	! Kentucky
!	Sdat(42)%region = 1	! Tennessee
!	Sdat(48)%region = 1	! West Virginia

! Mountain
!	Sdat( 6)%region = 1	! Colorado
!	Sdat(44)%region = 1	! Utah

! Mountain North
!	Sdat(12)%region = 1	! Idaho
!	Sdat(26)%region = 1	! Montana
!	Sdat(50)%region = 1	! Wyoming

! South Central
!   Sdat(36)%region = 1	! Oklahoma
!   Sdat(43)%region = 1	! Texas

! Southwest
!	Sdat( 3)%region = 1	! Arizona
!	Sdat(28)%region = 1	! Nevada
!	Sdat(31)%region = 1	! New Mexico

!	Sdat( 2)%region = 1	! Alaska
!	Sdat(11)%region = 1	! Hawaii  -- no observations

! New England
!	Sdat( 7)%region1 = 1	! Connecticut
!	Sdat(19)%region1 = 1	! Maine
!	Sdat(21)%region1 = 1	! Massachusetts
!	Sdat(29)%region1 = 1	! New Hampshire
!	Sdat(39)%region1 = 1	! Rhode Island
!	Sdat(45)%region1 = 1	! Vermont


! New England
!	Sdat( 7)%region1 = 1	! Connecticut
!	Sdat(19)%region1 = 1	! Maine
!	Sdat(21)%region1 = 1	! Massachusetts
!	Sdat(29)%region1 = 1	! New Hampshire
!	Sdat(39)%region1 = 1	! Rhode Island
!	Sdat(45)%region1 = 1	! Vermont

! East
!	Sdat( 8)%region2 = 1	! Delaware
!	Sdat(20)%region2 = 1	! Maryland
!	Sdat(30)%region2 = 1	! New Jersey
!	Sdat(32)%region2 = 1	! New York
!	Sdat(38)%region2 = 1	! Pennsylvania

! Southeast
!	Sdat( 9)%region3 = 1	! Florida
!	Sdat(10)%region3 = 1	! Georgia
!	Sdat(33)%region3 = 1	! North Carolina
!	Sdat(40)%region3 = 1	! South Carolina
!	Sdat(46)%region3 = 1	! Virginia

! Upper Midwest 
!	Sdat(22)%region4 = 1	! Michigan
!	Sdat(23)%region4 = 1	! Minnesota
!	Sdat(34)%region4 = 1	! North Dakota
!	Sdat(41)%region4 = 1	! South Dakota
!	Sdat(49)%region4 = 1	! Wisconsin

! Midwest 
!	Sdat(13)%region5 = 1	! Illinois
!	Sdat(14)%region5 = 1	! Indiana
!	Sdat(15)%region5 = 1	! Iowa
!	Sdat(16)%region5 = 1	! Kansas
!	Sdat(25)%region5 = 1	! Missouri
!	Sdat(27)%region5 = 1	! Nebraska
!	Sdat(35)%region5 = 1	! Ohio

! South
!	Sdat( 1)%region6 = 1	! Alabama
!	Sdat( 4)%region6 = 1	! Arkansas
!	Sdat(18)%region6 = 1	! Lousiana
!	Sdat(24)%region6 = 1	! Mississippi
	
! South Central
 !   Sdat(36)%region7 = 1	! Oklahoma
 !   Sdat(43)%region7 = 1	! Texas

! Appalachia
!	Sdat(17)%region8 = 1	! Kentucky
!	Sdat(42)%region8 = 1	! Tennessee
!	Sdat(48)%region8 = 1	! West Virginia

! Southwest
!	Sdat( 3)%region9 = 1	! Arizona
!	Sdat(28)%region9 = 1	! Nevada
!	Sdat(31)%region9 = 1	! New Mexico

! Mountain
!	Sdat( 6)%region10 = 1	! Colorado
!	Sdat(44)%region10 = 1	! Utah

! Mountain North
!	Sdat(12)%region10 = 1	! Idaho
!	Sdat(26)%region10 = 1	! Montana
!	Sdat(50)%region10 = 1	! Wyoming

!	Sdat( 5)%region11 = 1	! California

!	Sdat( 2)%region12 = 1	! Alaska

! Omitted

! Northwest
!	Sdat(37)%region = 1	! Oregon
!	Sdat(47)%region = 1	! Washington

!	Sdat(11)%region = 1	! Hawaii  -- no observations


!	scale population by 10s of millions
Sdat(:)%pop = Sdat(:)%pop*1.0e-7

NILOC = count(sloc==0) !number of selected states for home premium

!	build master matrix of ilocation
ma_iloc(:) = 0
j=1
do i=1,NLOC
   if (sloc(i) == 0) then
!			print *,"initial location ",i," is j=",j
      ma_iloc(j) = i    
      j=j+1
   end if
end do

allocate(qe(NPARME))           !number of parameters to estimate
allocate(ffe(NPARME)) 
allocate(qdir(NPARME,NPARME))  !needed to build basis for amoeba

if (myid .eq. master) then
   print '(A)',title
   print '(A)',"Estimation Results:  mbr93a.F90"
   print '(A,I10)',"Number of parameters to estimate:     ",NPARME
   print *," "
   print '(A,I10)',"Number of locations:                  ",NLOC
   print '(A,I10)',"Number of Selected locations(NILOC):  ",NILOC
   print *," "
   
	print '(A, I10)',"Number of  fixed  effect support points (wages):              ",NPT_fe
	print '(A,2I10)',"Number of location match support points (wages, preferences): ",NPT_lm,NPT_lm2
	print '(A,I10)' ,"Number of transient effect variance support points:           ",NPT_sig_eps
   
   print *," "
   print '(A,I10)',"Number of moving cost types:                                  ",Ntype
   print *," "
   print '(A,A)',"Input file:           ",trim(adjustl(in_fname))
   print '(A,A)',"Wage data file:       ",trim(adjustl(in_cdname))
   print '(A,A)',"Expected income file: ",trim(adjustl(wage_fname))
end if ! (myid .eq. master) 

call start_values

NoHome = 0
if (abs(q(p_home)) < 1.0e-15 .AND. ss(p_home)==0) then !no home premium
		NoHome = 1
   if (myid .eq. master) then
      print *, "No home premium"
      print *," "
   end if ! myid
end if

if (myid .eq. master) then
   
	print '(A,F10.5)',"BETA (not estimated):            ",BETA
	print *," "
	print *,"Scaling factor: ", IFAC
	print '(A,I10)',"Final Estimated Values (FEV):      ",FEV
	print *," "
   print *,"                             Age 20"
   print *," Loc Select     Population Mean Wage"
   print '(2I4,1X,E18.12,F12.5)',(loc(i),sloc(i),Sdat(i)%pop,sdat(i)%mu,i=1,NLOC)
   print *," "
   print *,"List of initial locations."
   print *,"index           ma_iloc" 
   print '(2I10)',(j,ma_iloc(j),j=1,NILOC)
   print *," "

	print *,"State wage censoring points"
	print *,"Scaling factor: ", IFAC
	print *,"Loc        wage" 
	print '(I10,f12.6)',(j,Sdat(j)%Ben,j=1,NLOC)
	print *," "

end if ! myid


NoPopPref = 0
if (abs(q(p_pop_pref)) < 1.0e-15 .AND. ss(p_pop_pref)==0) then !no population preference effect
   NoPopPref = 1
end if


!	initialize parameters

j = 1
do i=1,NPARMS
   if (ss(i) == 1) then 
       qe(j) = q(i)
      ffe(j) = ff(i)
      j=j+1
   end if
end do

NprobE = sum(ss(NPARMS-NTYPE+2:NPARMS))  ! number of estimated mixing probability parameters

if (myid .eq. master) then
   if (sum(ss) /= NPARME) then
      print *,"Error:: Mismatch between number of parameters to estimate and selection vector." 
      print *,"NPARME = ",NPARME
      print *,"Selection vector=",ss
      stop
   end if
end if ! myid

!	set initial direction vector
qdir = 0.0
forall (i=1:NPARME) qdir(i,i) = 1.0

CALL read_adj        !reads 51x51 matrix of adjacent States 

CALL read_climate()  !reads climate data 9 measures per State 
CALL fill_miles      !Calculate distance between States - needed for cost of moving
CALL read_data       !reads data from Unit 7 and fills mdat() structure


!
! finished setting up: now do something
!
!All processors to this point
!Now split tasks between master and slaves
if (myid .eq. master) then

   print *," "
   print '(A,I10)',   "Horizon Age:                                   ",CAP_AGE
   print '(A,E24.12)',"Newton Based convergence tolerance (loglike) : ",TOL2
   print '(A,E24.12)',"Maximum (proportional) Newton step           : ",MaxStep
   print *," "


   if (FEV == 0) then 
      print *," "
      print *,"FEV==0, Invoke Newton " 
      print '(A)'," Starting values for estimated parameters: "
      print *,qe
      !		need baseline function value
      fval1  = -Func(qe) !calculates f0 which is used in GIH called within Newton
      fval00 = fval1
      print *," Call Newton"

      do i=1,NewtonSteps 

!         print *," "
!         print '(A,3F15.6)', "  loglikelihood, wage component, migration component", fval1, ll_w, fval1-ll_w
         print *," "
         print *," Newton Step: ",i
         print *," "
         CALL Newton(qe,fval1)  !est parm vector; baseline loglikelihood value

         if ( abs(fval1-fval0) <= TOL2*(abs(fval0)+abs(fval1))) then 
            exit
         end if
         if (i == NewtonSteps) print *, "Reached maximal number of Newton iterations: ", NewtonSteps
         fval0 = fval1
      end do

      print '(A,I5)',"Number of Newton Steps: ",i
      print *," "
      
      print '(A)',title
      print *," "
      print '(A)'," Parameters values (1 if estimated): "
      print *," "
      
      call par_values

      print '(A,2(/,f24.10))', " Initial and final loglikelihood values ", fval00,fval1
      print *, " chi-square                             ", 2*(fval1-fval00)
      do i=1,numprocs-1
         call MPI_SEND(-1,1,MPI_INTEGER,i,0,MPI_COMM_WORLD,ierr)
      end do

   else if (FEV /= 0) then
      
      print '(A,E24.12)',"Amoeba Convergence tolerance (loglike):       ",TOL1
      print '(A,/,3(3E12.4,1X))',"Initial step sizes for amoeba(estimation): ",ff

      !	build basis and set up y
      allocate(pt(NPARME+1,NPARME))
      allocate(y(NPARME+1))
      print *,"Build basis for amoeba."
      pt(1,:) = qe
      print *,"First point (starting values): "
      print *,pt(1,:)
      !	need to be consistent with Amoeba, define y as -loglike
      y(1) = Func( pt(1,:) ) 
      print *,"(main) Loglikelihood evaluation: ",-y(1)
      print *," "
      j = 2
      do i=1,NPARMS
         if (ss(i) == 1) then
            if (ff(i) == 0) then
               print *,"ERROR: ff(",i,") is zero, must be nonzero."
               print *,"Change control deck and restart."
               CALL MPI_ABORT(MPI_COMM_WORLD,-1,ierr)
               stop
            end if
            pt(j,:) = pt(1,:) + qdir(j-1,:)*ff(i)
            print '(A,I2,/,3(3E24.12,/))',"point ",j,pt(j,:)
            y(j) = Func(pt(j,:)) !to be consistent with Amoeba, y = -loglike
            print *,"Loglikelihood evaluation: ",-y(j)
            print *," "
            j=j+1
         end if
      end do
      print *," "
      print *,"Vectors loaded and calling amoeba."
      
      Call amoeba(pt,y,TOL1,Func,iters)
      
      print *,"Returned from amoeba."
      print *,"Number of function evaluations: ",iters
      print *," "
      print *,"Parameter estimates: "
      print *,pt(1,:)
      print *," "
      print *,"Loglikelihood value: ",-y(1)
      
      print *," "
      fval = -y(1)
      qe = pt(1,:)
      print '(A,F24.16)',"Loglikelihood value: ",fval
      print *,"Number of iterations: ",iters
      print *,"Estimated parameters: "
      print '(3E24.12,/,3E24.12,/,3E24.12)',qe
      print *," "
      fval1 = -Func(qe) !calculates f0 which is used in  Newton
      fval0    = fval  !watch signs
      fval1    = fval
      flag = 0
      
      do iters=1,FEV   !FEV is the Maximum number of Newton Steps
         print *," "
         print *," (main) Newton Iteration: ",iters
         Call Newton(qe,fval1) !returns new value of loglike
         if ( abs(fval1-fval0) <= TOL2*(abs(fval0)+abs(fval1))) then 
            flag = 1
            exit
         end if
         fval0 = fval1
      end do
      if (flag == 1) then
         print '(A,I5/)',"Converged. Number of Newton Steps: ",iters
      else
         print *,"Maximum Number of Newton Steps Taken."
      end if
      
      call par_values
      
      print *,"Final value of loglikelihood function = ",fval1
      !send hloc = -1 tell slaves to stop working
      CALL MPI_BCAST(-1,1,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
      
   end if !FEV==0
   
else !slave
   
   Call Slave_work 
   
end if !myid master

CALL MPI_FINALIZE(ierr)

end program mbr93a
