module mbr_vars
! use nrtype
IMPLICIT NONE


	Integer , parameter :: MAXREC = 100000
	Integer , parameter :: IDmax  = 100		! maximal number of replicas per individual (simulated data)
	Integer , parameter :: MvLoc  =  7		! maximal number of locations visited by each individual
	Integer , parameter :: NPARMS = 87		! total number of parameters

	Integer, parameter ::  NLOC        = 50					! DC treated as if it is a part of MD
	Integer, parameter ::  NPT_fe_par  = 3 					! support points for individual fixed effect 
	Integer, parameter ::  NPT_lm      = 3					! location match component of wages
	Integer, parameter ::  NPT_lm2     = 3					! location match component of preferences
	Integer, parameter ::  NPT_sig_eps = 4 					! support points for transient effect mixture

	Integer, parameter ::  NPT_fe      = 2*NPT_fe_par+1 	! support points for individual fixed effect 

	Integer, parameter ::  Ntype   =  3
	Integer, parameter ::  MaxT_i  = 15	! maximal number of observations per individual

	Integer, parameter ::  NBINS  = NPT_lm**MvLOC
	Integer, parameter ::  NBINS2 = NPT_lm2**MvLOC

	Integer            ::  CAP_AGE   = 60  !
	Integer, parameter ::  MAX_AGE   = 60  !maximal array size
	Integer, parameter ::  LAST_AGE  = 36  !LL contribution
	Integer, parameter ::  FIRST_AGE = 20  !start of work period

	Integer            ::  NREPS = 10           !# of reps per itype in simulations
	Integer            ::  FEV				    ! 0 for Newton, positive for Nelder-Mead
    Integer            ::  NewtonSteps = 200    ! Maximum number of Newton iterations

	Double Precision :: BETA	! read from control deck; not estimated

	Integer             :: NPARME      ! number of parameters to estimate
	Integer             :: NprobE	! number of estimated mixing probability parameters

	Double Precision ,     dimension(NPARMS) :: q			!contains fixed and est parameters
	Double Precision ,     dimension(NPARMS) :: std = -1.0  !estimated standard errrors (-1 means not estimated)
	Integer , dimension(NPARMS) :: ss			!selection of est parameters
    Double Precision, dimension(NPARMS) :: ff   !scale factors to build basis for amoeba
    Double Precision, dimension(:),   allocatable :: ffe		! scale factors for estimated parameters

    
	Integer, dimension(Ntype) :: costtype ! index of cost type parameters
	Integer, dimension(Ntype) :: probtype ! index of type probability parameters

	Double Precision  , dimension(Ntype) :: ptype ! vector of type probabilities (may be estimated)
	
	Double Precision,     parameter :: EVFTOL = 1.0e-6	!tolerance in calculate FP

	Double Precision , parameter ::    TINY = 1.0e-20
	Real (Selected_Real_Kind (8,100)) , parameter ::   TINY2 = 1.0e-37 ! this is apparently as small as it gets, in the unix compiler (Kind=4)
	Double Precision  :: MaxStep = 1.0d0		! biggest proportional change allowed in Newton step

	Double Precision , parameter ::	TOL1 = 1.0e-5  !used to determine convergence of Nelder-Mead algorithm
	Double Precision , parameter ::	TOL2 = 1.0e-8  !used to determine convergence of Newton algorithm

	Character(LEN=80) :: in_fname, in_cdname, wage_fname,adj_fn, amenity_fn, latlong_fn	! input file, mean wages by age and State, expected incomes, adjacent States
!	Character(LEN=80) :: cdeck = "~/mbr85/cdeck.mbr85"	    ! control deck file name
	Character(LEN=80) :: cdeck = "/home/jkennan/mbr93a/cdeck.mbr93a"	! control deck file name (Palah wants the full name)

	Type record_type
		Integer :: iloc, Nobs_i, Nloc_i,        Nbins_i, Nbins2_i, afqt
		Logical :: Moved				! Moved indicates whether this person ever moved
!		Integer :: id, iloc, Nobs_i, Nloc_i, Moved, Nbins_i, Nbins2_i, afqt	! Moved indicates whether this person ever moved
!		Integer         , dimension(MaxT_i) :: yr, csta, psta, dsta, age
		Integer         , dimension(MaxT_i) ::     csta, psta, dsta, age, year
		Integer         , dimension(MaxT_i) :: loci_index  = 0			! pointer to vloci_list for  current location
		Integer         , dimension(MaxT_i) :: ploci_index = 0			! pointer to vloci_list for previous location
		Integer         , dimension(MvLoc)  :: vloci_list  = 0			! list of locations visited by this id
!		Double Precision, dimension(MaxT_i) :: lwage  = 0.0				! logwage
		Real			, dimension(MaxT_i) ::  wage  = 0.0				!    wage
		Double Precision, dimension(MaxT_i,Npt_lm,Npt_lm2,0:Npt_lm,0:Npt_lm2) :: vv = 0.0 ! choice probabilities (obs_t,cbin,cbin2,pbin,pbin2)
!		Double Precision, dimension(NPT_lm**MVLOC) :: q_i				! probability of the given sequence of wage bins
	end type record_type

   Type State_type
		Integer  :: State_num, seashore
		Integer , dimension(NLOC) :: Region = 0
		Double Precision :: heat_dd, cool_dd, total_dd, win_temp,sum_temp,mean_temp,dsw_temp,precip,pop, lbenefit, Ben
		Double Precision  :: mu		!mean wage at age 20 (from pums)
		Character(len=14) :: name
   end type State_type

	Double Precision :: IFAC   !scale income , read from control deck

	Integer :: NILOC 			! # of initial locations
	Integer :: Nobs_TOT , N_i  ! person-years, persons
	Integer :: NoHome			! no home premium
	Integer :: NoPopPref		! population does not enter through preference shocks

!	initial locations
	Integer , dimension(NLOC) :: ma_iloc !master list of initial locations
 
!	utility and EV function
	Double Precision,              dimension(NLOC,0:NPT_lm,0:NPT_lm2,0:NLOC,0:NPT_lm,0:NPT_lm2)	     :: EV !(csta,cb,cb2,psta,pb,pb2)
	Double Precision, allocatable, dimension(   :,       :,        :,     :,       :,        :,   :) :: v  !(csta,cb,cb2,psta,pb,pb2,dsta)
	Double Precision,              dimension(NLOC,  NPT_lm,  NPT_lm2,0:NLOC,                   NLOC) :: u  !(csta,cb,cb2,psta,         dsta)

!	function evaluations for standard error calculation
	Double Precision, allocatable, dimension(:,:) :: f00		! individual likelihood, by type (used to compute gradient for type probability parameters)
	Double Precision, allocatable, dimension(:)   :: f0			! individual loglikelihood
	Double Precision, allocatable, dimension(:)   :: f0save		! individual loglikelihood
	Double Precision, allocatable, dimension(:)   :: L0			! individual    likelihood

	Double Precision :: L_w_i, L_w , ll_w, Like_iw0			! likelihood and loglikelihood of observed wages

	Type(Record_type) , allocatable, dimension(:)  :: dat !	master data file

	Type( State_type) , save,  dimension(NLOC) :: Sdat !	state characteristics

!	Setup matrix to hold distance between locations
	Double Precision, allocatable, dimension(:,:)  :: miles


	Character(LEN=60)           :: sim_fname       !name of simulation file

	Integer , dimension(MVLOC,NBINS)  :: spt   ! support point indices
	Integer , dimension(MVLOC,NBINS2) :: spt2  ! support point indices

	Integer , dimension(NLOC,NLOC) :: adjacent 

	Double Precision, dimension(NPT_fe)	:: eta      = 0.0	 ! support points for fixed effects  distribution
	Double Precision, dimension(NPT_lm)	:: upsilon  = 0.0	 ! support points for location match distribution (wages)
	Double Precision, dimension(NPT_lm2):: upsilon2 = 0.0	 ! support points for location match distribution (preferences)
	Double Precision, dimension(NPT_fe)	:: PP1  = 1.0d0/NPT_fe	 ! support point probabilities
	Double Precision, dimension(NBINS)	:: Pcb  = 1.0d0/NBINS	 ! support point probabilities
	Double Precision, dimension(NBINS2)	:: Pcb2 = 1.0d0/NBINS2 ! support point probabilities

	Double Precision :: mu0

	Double Precision, dimension(NPT_sig_eps)  :: PP_sig_eps = 1.0d0/NPT_sig_eps	! transient variances
	Double Precision, dimension(NPT_sig_eps)  :: sig_eps						! transient variances

	Double Precision :: w00   = - 10.0 ! treat (scaled) wage observations below this as missing
	Double Precision :: lli00 = -100.0 ! loglikelihood for observations that are almost impossible (L(obs_i) < TINY2)

	Integer, parameter  :: master=0
	Integer :: myid, numprocs

	Integer :: maxage, hloc
	Integer :: pwflag = 1
	Integer :: plflag = 0

!	location of parameters in q() vector (set up in the start_values routine)
	Integer :: 	p_cost1	,p_distance,p_adjacent,	p_home, p_previous,	p_age,	p_pop,	p_cool, p_heat
	Integer ::	p_income, p_meanwage, p_mu, p_year, p_linearwage, p_quadwage, p_afqt, p_afqt_age, p_sigups, p_sigeps1, p_sigeps2, p_sigeps3, p_sigeps4
	Integer ::	p_eta1, p_eta2, p_eta3, p_sigups2, p_cost3, p_cost2, p_sea, p_rain, p_degdays,  p_pop_pref
	Integer ::	p_movehome, p_homedistance, p_probtype2, p_probtype3

	Integer , dimension(NLOC)::	p_region

!	Scaling of age and AFQT scores
	Real, parameter ::	a0  = 20.0
	Real, parameter ::	ab0 = 50.0


end module mbr_vars
