MODULE nrutil 
!use nrtype

Interface imaxloc
	Module procedure imaxloc_r, imaxloc_i
end interface

Interface outerprod
	Module procedure outerprod_d
end interface

Interface swap
	Module procedure swap_i, swap_r, swap_rv, &
		masked_swap_rs, masked_swap_rv,masked_swap_rm
end interface
	
Interface assert_eq
	Module procedure assert_eq2, assert_eq3, assert_eq4, assert_eqn
end interface

CONTAINS
	Function assert_eq2(n1,n2,string)
		!Report and die if integers not all equal (used for size checking)
		Character(LEN=*), intent(in) :: string
		Integer , intent(in) :: n1, n2
		Integer :: assert_eq2

		if (n1==n2) then 
			assert_eq2 = n1
		else
			write(*,*) 'nerror: an assert_eq failed with this tag: ',string
			STOP 'program terminated by assert_eq2'
		end if
	end Function assert_eq2

	Function assert_eq3(n1,n2,n3,string)
		Character(LEN=*), intent(in) :: string
		Integer , intent(in) :: n1, n2, n3
		Integer :: assert_eq3

		if (n1 == n2 .AND. n2 == n3) then
			assert_eq3 = n1
		else
			write(*,*) 'nerror: an assert_eq failed with this tag: ',string
			STOP 'program terminated by assert_eq3'
		end if
	end Function assert_eq3
	
	Function assert_eq4(n1,n2,n3,n4,string)	
		Character(LEN=*), intent(in) :: string
		Integer , intent(in) :: n1, n2, n3, n4
		Integer :: assert_eq4

		if (n1 == n2 .AND. n2 == n3 .AND. n3 == n4) then
			assert_eq4 = n1
		else
			write(*,*) 'nerror: an assert_eq failed with this tag: ',string
			STOP 'program terminated by assert_eq4'
		end if
	end Function assert_eq4

	Function assert_eqn(nn,string)
		Character(LEN=*), intent(in) :: string
		Integer , intent(in), dimension(:) :: nn
		Integer :: assert_eqn

		if ( all(nn(2:) == nn(1)) ) then
			assert_eqn = nn(1)
		else
			write(*,*) 'nerror: an assert_eq failed with this tag: ',string
			STOP 'program terminated by assert_eqn'
		end if
	end Function assert_eqn

	Function ifirstloc(mask)
		!index of first occurrence of .true. in a logical vector
		Logical , dimension(:), intent(in) :: mask
		Integer , dimension(1) :: loc
		Integer :: ifirstloc
		loc=maxloc(merge(1,0,mask))
		ifirstloc = loc(1)
		if (.not. mask(ifirstloc)) ifirstloc = size(mask)+1
	end Function ifirstloc

	Function imaxloc_r(arr)
		!index of maxloc on an array
		Double Precision, dimension(:), intent(in) :: arr
		Integer :: imaxloc_r
		Integer, dimension(1) :: imax
		imax=maxloc(arr(:))
		imaxloc_r = imax(1)
	end Function imaxloc_r

	Function imaxloc_i(iarr)
		Integer , dimension(:), intent(in) :: iarr
		Integer :: imaxloc_i
		Integer , dimension(1) :: imax
		imax = maxloc(iarr(:))
		imaxloc_i = imax(1)
	end Function imaxloc_i

	Function iminloc(arr)
		!index of minloc of an array
		Double Precision, dimension(:), intent(in) :: arr
		Integer , dimension(1) :: imin
		Integer :: iminloc
		imin = minloc(arr(:))
		iminloc = imin(1)
	end Function iminloc
	
	Subroutine nerror(string)
		Character(LEN=*), intent(in) :: string

		write(*,*) 'nerror:  ',string
		STOP 'Program terminated by nerror.'
	end SUBROUTINE nerror

	Function outerprod_d(a,b)
		Double Precision, dimension(:), intent(in) :: a,b
		Double Precision, dimension(size(a),size(b)) :: outerprod_d
		outerprod_d = spread(a,dim=2,ncopies=size(b)) * &
			spread(b,dim=1,ncopies=size(a))
	end Function outerprod_d

	Subroutine swap_i(a,b)
		Integer , intent(inout) :: a,b
		Integer  :: dum
		dum=a
		a=b
		b=dum
	end SUBROUTINE swap_i

	SUBROUTINE swap_r(a,b)
		Double Precision , intent(inout) :: a,b
		Double Precision :: dum
			dum = a
			a = b
			b = dum
	end SUBROUTINE swap_r

	SUBROUTINE swap_rv(a,b)
		Double Precision, dimension(:), intent(inout) :: a,b
		Double Precision, dimension(Size(a)) :: dum
			dum=a
			a = b
			b = dum
	end SUBROUTINE swap_rv

	SUBROUTINE masked_swap_rs(a,b,mask)
		Double Precision, intent(inout) :: a,b
		Logical, intent(in)     :: mask
		Double Precision :: swp
		if (mask) then
			swp = a
			a = b
			b = swp
		end if
	end SUBROUTINE masked_swap_rs

	SUBROUTINE masked_swap_rv(a,b,mask)
		Double Precision, dimension(:), intent(inout) :: a,b
		Logical,  dimension(:), intent(in)    :: mask
		Double Precision , dimension(Size(a))         :: swp
		where (mask)
			swp = a
			a = b
			b = swp
		end where
	end SUBROUTINE masked_swap_rv

	SUBROUTINE masked_swap_rm(a,b,mask)
		Double Precision, dimension(:,:), intent(inout)  :: a,b
		Logical , dimension(:,:), intent(in)     :: mask
		Double Precision, dimension(Size(a,1),Size(a,2)) :: swp
		where (mask)
			swp = a
			a = b
			b = swp
		end where
	end SUBROUTINE masked_swap_rm
end MODULE nrutil 
